@@ -7,15 +7,11 @@ library(xtable)
 
 if(!requireNamespace("ggtree")) {
   message("Building the vignette requires ggtree R-package. Trying to install.")
-  status.ggtree <- try({
+  try({
     if (!requireNamespace("BiocManager", quietly = TRUE))
       install.packages("BiocManager")
     BiocManager::install("ggtree", version = "3.9")
   }, silent = TRUE)
-  if(class(status.ggtree) == "try-error") {
-    stop(
-      "The ggtree installation did not succeed. The vignette cannot be built.")
-  }
 }
 
 options(digits = 2)
@@ -45,7 +41,7 @@ X <- cbind(
 colnames(X) <- as.character(1:5)
 
 ## ----PlotTreeAndData, include=FALSE, eval=FALSE-------------------------------
-#  library(tikzDevice); library(ggtree); library(ggplot2); library(data.table);
+#  library(tikzDevice); library(ggplot2); library(data.table);
 #  # 4. Plotting the tree, the data and the active coordinate vectors:
 #  tipValueLabels <- data.table(
 #    node = seq_len(PCMTreeNumTips(tree)),
@@ -68,16 +64,28 @@ colnames(X) <- as.character(1:5)
 #  
 #  tikz(file = "TreeMGPMExample.tex", width = 8, height = 5)
 #  palette <- PCMColorPalette(2, names = c("1", "2"), colors = c("black", "orange"))
-#  plTree <- PCMTreePlot(tree, palette = palette, size=2) +
-#    geom_nodelab(geom = "label", color = "red") + geom_tiplab(geom = "label", color = "black")
-#  plTree <- plTree %<+% tipValueLabels %<+% dtNodes[, list(node = endNode, kLabel, kLabel2, tLabel)]
-#  plTree +
-#    geom_tiplab(geom = "text", aes(label = valueLabel), color = "black", hjust = -0.2, vjust = -1.1) +
-#    geom_tiplab(geom = "text", aes(label = kLabel), color = "black", hjust = -0.4, vjust = 1.1) +
-#    geom_nodelab(geom = "text", aes(label = kLabel), color = "red", hjust = -0.2, vjust = 0.6) +
-#    geom_nodelab(geom = "text", aes(label = kLabel2), color = "red", hjust = 0.4, vjust = 2.8) +
-#    geom_text(aes(x = branch, label = tLabel), vjust = -0.8, color = "black") +
-#    scale_x_continuous(limits = c(0, 5.2)) + scale_y_continuous(limits = c(0.8, 5.2))
+#  
+#  plTree <- PCMTreePlot(tree, palette = palette, size=2)
+#  
+#  # Plot the tree with branches colored according to the regimes.
+#  # The following code works correctly only if the ggtree package is installed,
+#  # which is not on CRAN.
+#  if(requireNamespace("ggtree")) {
+#    require(ggtree)
+#  
+#    plTree <- plTree + geom_nodelab(geom = "label", color = "red") + geom_tiplab(geom = "label", color = "black")
+#    plTree <- plTree %<+% tipValueLabels %<+% dtNodes[, list(node = endNode, kLabel, kLabel2, tLabel)]
+#    plTree <- plTree +
+#      geom_tiplab(geom = "text", aes(label = valueLabel), color = "black", hjust = -0.2, vjust = -1.1) +
+#      geom_tiplab(geom = "text", aes(label = kLabel), color = "black", hjust = -0.4, vjust = 1.1) +
+#      geom_nodelab(geom = "text", aes(label = kLabel), color = "red", hjust = -0.2, vjust = 0.6) +
+#      geom_nodelab(geom = "text", aes(label = kLabel2), color = "red", hjust = 0.4, vjust = 2.8) +
+#      geom_text(aes(x = branch, label = tLabel), vjust = -0.8, color = "black") +
+#      scale_x_continuous(limits = c(0, 5.2)) + scale_y_continuous(limits = c(0.8, 5.2))
+#  }
+#  
+#  plTree
+#  
 #  dev.off()
 
 ## ---- results='asis'----------------------------------------------------------
